/* SkyData.c - for linux
   (C) 1999 Diego Picciani - NOVA COMPUTER s.r.l.
   All Rights Reserved

   THIS IS NOT FREE SOFTWARE

   THERE ARE NOT ANY KIND OF WARRANTY, EITHER IMPLICIT OR
   EXPLICIT for any kind of damage and/or malfunction
   and/or inhability to use the system, that could be derivated
   from the use of this software.


   This code was slightly modified by Kang-Hyuk Lee (kdean) at Telemann
	 under the permission of the Picciani.
	
   Skymedia is a Trademark of Telemann
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include "sm2_lib.h"

#define MAX_PATH_LEN	1023

#define INI_FILE			"SkyData.ini"
#define MAX_PIDS			30
#define MIN_PIDMODE		1
#define MAX_PIDMODE		5
#define MIN_FREQ			10000
#define MAX_FREQ			13000
#define MIN_SR				1000
#define MAX_SR				30000
#define MIN_LO				9000
#define MAX_LO				11000
#define MIN_TB				0
#define MAX_TB				2
#define MIN_DISEQC		0
#define MAX_DISEQC		16

#define VERSION				"2.01"

char	program_path[MAX_PATH_LEN + 1] = { 0 };
float	frequency,symbolrate,localoscillator;
int		alignment,lnbmode,lnbpower,toneburst,diseqcport,ipaddr[4] = { 10,10,10,10 };
int		datapid[MAX_PIDS],datamode[MAX_PIDS],qtpids,skyStart,skyOpen,macfilter,skyPIDs;

void Error(char *fmt,...)
{
	va_list							arg;
	char								app[1024];
	int									i;
	SETDATA_STRUCTURE		pid;
	
	va_start(arg,fmt);
	vsprintf(app,fmt,arg);
	printf("%s\n",app);
	if(skyPIDs)
		for(i = 0;i < qtpids;i++)
		{
			pid.Encapsulation = datamode[i];
			pid.DataPID = datapid[i];
			pid.TransportID = 0;
			pid.pgnumber = 0;
			pid.PmtPID = 0;
			Sky_ReleaseNetworkPID(pid);
		}
	if(skyStart)
		Sky_StopSystem();
	if(skyOpen)
		Sky_CloseSystem();
	exit(1);
}

void GetPaths(char *fullpath)
{
	char		*bar;
	
	if(strlen(fullpath) > (MAX_PATH_LEN - 10))
		Error("Working PATH too long !");
	strcpy(program_path,fullpath);
	if((bar = strrchr(program_path,'/')) == NULL)
		strcpy(program_path,".");
	else
		*bar = 0;
}

void RemoveSpaces(char *str)
{
	int		i;
	
	for(i = 0;str[i] == ' ' || str[i] == '\t';i++);
	if(i)
		strcpy(str,&str[i]);
	for(i = strlen(str) - 1;i >= 0 && (str[i] == ' ' || str[i] == '\t');i--);
	str[i + 1] = 0;
}

void LoadConfig(void )
{
	FILE	*fp;
	char	fname[MAX_PATH_LEN + 32],line[1024],*sep,*sep2,*endptr;
	int		donefrequency,donesymbolrate,donelocaloscillator,doneip,donemacfilter;
	int		donealignment,donelnbmode,donelnbpower,donetoneburst,donediseqcport;
	int		idline,i;
	
	qtpids = 0;
	donefrequency = donesymbolrate = donelocaloscillator = donemacfilter = doneip = 0;
	donealignment = donelnbmode = donelnbpower = donetoneburst = donediseqcport = 0;
	sprintf(fname,"%s/%s",program_path,INI_FILE);
	if((fp = fopen(fname,"rb")) == NULL)
		Error("Cannot open %s",fname);
	for(idline = 0;;)
	{
		line[0] = 0;
		fscanf(fp,"%[^\n]s",line);
		getc(fp);
		if(feof(fp) && !line[0])
			break;
		idline++;
		RemoveSpaces(line);
		if(line[0] == 0 || line[0] == '#')
			continue;
		if((sep = strchr(line,'=')) == NULL)
			Error("Syntax error in %s line %d",fname,idline);
		*sep = 0;
		sep++;
		RemoveSpaces(line);
		if(!strcasecmp(line,"Frequency"))
		{
			if(donefrequency)
				Error("Frequency redefined in %s line %d",fname,idline);
			RemoveSpaces(sep);
			frequency = atof(sep);
			if(frequency < MIN_FREQ || frequency > MAX_FREQ)
				Error("Frequency out of limits in %s line %d",fname,idline);
			donefrequency = 1;
		}
		else if(!strcasecmp(line,"SymbolRate"))
		{
			if(donesymbolrate)
				Error("SymbolRate redefined in %s line %d",fname,idline);
			RemoveSpaces(sep);
			symbolrate = atof(sep);
			if(symbolrate < MIN_SR || symbolrate > MAX_SR)
				Error("SymbolRate out of limits in %s line %d",fname,idline);
			donesymbolrate = 1;
		}
		else if(!strcasecmp(line,"LocalOscillator"))
		{
			if(donelocaloscillator)
				Error("LocalOscillator redefined in %s line %d",fname,idline);
			RemoveSpaces(sep);
			localoscillator = atof(sep);
			if(localoscillator < MIN_LO || localoscillator > MAX_LO)
				Error("LocalOscillator out of limits in %s line %d",fname,idline);
			donelocaloscillator = 1;
		}
		else if(!strcasecmp(line,"Alignment"))
		{
			if(donealignment)
				Error("Alignment redefined in %s line %d",fname,idline);
			RemoveSpaces(sep);
			if(strlen(sep) != 1 || (sep[0] != 'V' && sep[0] != 'v' && sep[0] != 'H' && sep[0] != 'h'))
				Error("Invalid Alignment in %s line %d",fname,idline);
			alignment = (sep[0] == 'h' || sep[0] == 'H') ? SIGNAL_HORIZONTAL : SIGNAL_VERTICAL;
			donealignment = 1;
		}
		else if(!strcasecmp(line,"LNBHigh"))
		{
			if(donelnbmode)
				Error("LNBHigh redefined in %s line %d",fname,idline);
			RemoveSpaces(sep);
			lnbmode = atoi(sep);
			if(lnbmode < 0 || lnbmode > 1)
				Error("Invalid LNBHigh in %s line %d",fname,idline);
			lnbmode = lnbmode ? LNB_HIGH : LNB_LOW;
			donelnbmode = 1;
		}
		else if(!strcasecmp(line,"LNBPower"))
		{
			if(donelnbpower)
				Error("LNBPower redefined in %s line %d",fname,idline);
			RemoveSpaces(sep);
			lnbpower = atoi(sep);
			if(lnbpower < 0 || lnbpower > 1)
				Error("Invalid LNBPower in %s line %d",fname,idline);
			lnbpower = lnbpower ? LNB_POWER_ON : LNB_POWER_OFF;
			donelnbpower = 1;
		}
		else if(!strcasecmp(line,"ToneBurst"))
		{
			if(donetoneburst)
				Error("ToneBurst redefined in %s line %d",fname,idline);
			RemoveSpaces(sep);
			toneburst = atoi(sep);
			if(toneburst < MIN_TB || toneburst > MAX_TB)
				Error("ToneBurst out of limits in %s line %d",fname,idline);
			switch(toneburst)
			{
				case 1:		toneburst = TONE_BURST_A;				break;
				case 2:		toneburst = TONE_BURST_B;				break;
				default:	toneburst = TONE_BURST_DISABLE;	break;
			}
			donetoneburst = 1;
		}
		else if(!strcasecmp(line,"DiseqcPort"))
		{
			if(donediseqcport)
				Error("DiseqcPort redefined in %s line %d",fname,idline);
			RemoveSpaces(sep);
			diseqcport = atoi(sep);
			if(diseqcport < MIN_DISEQC || diseqcport > MAX_DISEQC)
				Error("DiseqcPort out of limits in %s line %d",fname,idline);
			switch(diseqcport)
			{
				case 1:		diseqcport = DISEQC_PORT1;		break;
				case 2:		diseqcport = DISEQC_PORT2;		break;
				case 3:		diseqcport = DISEQC_PORT3;		break;
				case 4:		diseqcport = DISEQC_PORT4;		break;
				case 5:		diseqcport = DISEQC_PORT5;		break;
				case 6:		diseqcport = DISEQC_PORT6;		break;
				case 7:		diseqcport = DISEQC_PORT7;		break;
				case 8:		diseqcport = DISEQC_PORT8;		break;
				case 9:		diseqcport = DISEQC_PORT9;		break;
				case 10:	diseqcport = DISEQC_PORT10;		break;
				case 11:	diseqcport = DISEQC_PORT11;		break;
				case 12:	diseqcport = DISEQC_PORT12;		break;
				case 13:	diseqcport = DISEQC_PORT13;		break;
				case 14:	diseqcport = DISEQC_PORT14;		break;
				case 15:	diseqcport = DISEQC_PORT15;		break;
				case 16:	diseqcport = DISEQC_PORT16;		break;
				default:	diseqcport = DISEQC_DISABLE;	break;
			}
			donediseqcport = 1;
		}
		else if(!strcasecmp(line,"DataPID"))
		{
			if(qtpids >= MAX_PIDS)
				Error("Too many DataPID in %s line %d",fname,idline);
			if((sep2 = strchr(sep,',')) == NULL)
				Error("Syntax error in %s line %d",fname,idline);
			RemoveSpaces(sep);
			RemoveSpaces(sep2);
			datapid[qtpids] = (int)strtoul(sep,&endptr,16);
			datamode[qtpids] = atoi(sep2);
			if(datamode[qtpids] < MIN_PIDMODE || datamode[qtpids] > MAX_PIDMODE)
				Error("DataPID Mode of limits in %s line %d",fname,idline);
			qtpids++;
		}
		else if(!strcasecmp(line,"MACFilter"))
		{
			if(donemacfilter)
				Error("MACFilter redefined in %s line %d",fname,idline);
			RemoveSpaces(sep);
			macfilter = atoi(sep);
			if(macfilter < 0 || macfilter > 1)
				Error("Invalid MACFilter in %s line %d",fname,idline);
			donemacfilter = 1;
		}
		else if(!strcasecmp(line,"IPAddress"))
		{
			if(doneip)
				Error("IPAddress redefined in %s line %d",fname,idline);
			RemoveSpaces(sep);
			for(i = 0;i < 4;i++)
			{
				sep2 = strchr(sep,'.');
				if((sep2 != NULL && i == 3) || (sep2 == NULL && i != 3))
					Error("IPAddress error in %s line %d",fname,idline);
				if(sep2)
					*sep2 = 0;
				ipaddr[i] = atoi(sep);
				if(ipaddr[i] < 0 || ipaddr[i] > 255)
					Error("IPAddress error in %s line %d",fname,idline);
				sep = sep2 + 1;
			}
			doneip = 1;
		}
		else
			Error("Command %s not understood in %s line %d",line,fname,idline);
	}
	fclose(fp);
	if(!donefrequency)
		Error("Frequency does not appear in %s",fname);
	if(!donesymbolrate)
		Error("SymbolRate does not appear in %s",fname);
	if(!donelocaloscillator)
		Error("LocalOscillator does not appear in %s",fname);
	if(!donealignment)
		Error("Alignment does not appear in %s",fname);
	if(!donelnbmode)
		Error("LNBHigh does not appear in %s",fname);
	if(!donelnbpower)
		Error("LNBPower does not appear in %s",fname);
	if(!donetoneburst)
		Error("ToneBurst does not appear in %s",fname);
	if(!donediseqcport)
		Error("DiseqcPort does not appear in %s",fname);
	if(qtpids == 0)
		Error("At least one DataPID must appear in %s",fname);
	if(!donemacfilter)
		Error("MACFilter does not appear in %s",fname);
	if(!doneip)
		Error("IPAddress not defined, but MACFilter = 0 in %s",fname);
}

void SigHandler(int unused)
{
	Error("Signal Caught .... Exiting");
}

int main(int argc,char *argv[])
{
	char										mode[256];
	int											i;
	LNB_STRUCTURE						newLNB;
	TR_STRUCTURE						newTR;
	DATAPROGRAM_STRUCTURE		newDP;
	TUNER_RESULT						tunerResult;
	IP_ADDRESS							ip;
	IP_OR_MAC_FILTERING			ipormac;
	MAC_ADDRESS							macaddress;
	VERSION_NUM							driverversion,firmwareversion;
	SETDATA_STRUCTURE				pid;
	
	unsigned char curTunerStatus, prevTunerStatus;
	char strTunerStatus [2][10] = {"UnLocked", "Locked"}; 
	unsigned long cntTime = 0;	

	printf("SkyData V.%s started. (C) 1999 Diego Picciani\n",VERSION);
	printf("E-Mail: d.picciani@novacomp.it\n");
	signal(SIGINT,SigHandler);
	signal(SIGQUIT,SigHandler);
	signal(SIGTERM,SigHandler);
	skyStart = skyOpen = skyPIDs = 0;
	if(argc < 1)
		Error("Invalid Command Line !");
	GetPaths(argv[0]);
	LoadConfig();
	skyStart = skyOpen = 0;
	if(Sky_OpenSystem(DATA_APP))
		Error("Sky_OpenSystem Error");
	skyOpen = 1;
	if(Sky_StartSystem())
		Error("Sky_StartSystem Error");
	skyStart = 1;
	if(Sky_GetFirmwareVersion(&firmwareversion))
		Error("Sky_GetFirmwareVersion Error");
	if(Sky_GetDriverVersion(&driverversion))
		Error("Sky_GetDriverVersion Error");
	if(Sky_FreeTransponderStructure() != FALSE)
		Error("Sky_FreeTransponderStructure Error");
	printf("Skymedia Firmware Version : %d.%02d\n",firmwareversion.HighVersion,firmwareversion.LowVersion);
	printf("Skymedia Driver Version :   %d.%02d\n",driverversion.HighVersion,driverversion.LowVersion);
	skyOpen = 1;
	strcpy(newLNB.SatelliteName,"SkyData");
	newLNB.LocalOscillator = localoscillator * 1000.0;
	newLNB.SwitchOnOff = lnbmode;
	newLNB.LNBPower = lnbpower;
	newLNB.ToneBurst = toneburst;
	newLNB.DiseqcPort = diseqcport;
	if(Sky_SetNewLNBInformation(newLNB))
		Error("Sky_SetNewLNBInformation Error");
	newTR.TransponderNumber = 0;
	newTR.DownlinkFrequency = frequency * 1000.0;
	newTR.SymbolRate = symbolrate;
	newTR.SignalType = alignment;
	newTR.TransportID = 0;
	if(Sky_AddTransponder(&newTR))
		Error("Sky_AddTransponder Error");
	for(i = 0;i < qtpids;i++)
	{
		newDP.DataNumber = i;
		newDP.pgnumber = i;
		newDP.PmtPID = 0;
		newDP.DataPID = datapid[i];
		newDP.Encapsulation = datamode[i];
		newDP.SetFlag = TRUE;
		newDP.ManualAdd = TRUE;
		if(Sky_AddData(1,&newDP))
			Error("Sky_AddData Error");
	}
	printf("Requested Frequency :       %.3fGhz\n",frequency / 1000.0);
	printf("Requested Symbol Rate :     %.3fMSym/s\n",symbolrate);
	printf("Local Oscillator :          %.3fGhz\n",localoscillator / 1000.0);
	printf("Alignment :                 %s\n",alignment == SIGNAL_HORIZONTAL ? "Horizontal" : "Vertical");
	printf("LNB Mode :                  %s\n",lnbmode == LNB_HIGH ? "High" : "Low");
	printf("LNB Power :                 %s\n",lnbpower == LNB_POWER_ON ? "On" : "Off");
	if(toneburst == TONE_BURST_DISABLE)
		printf("Tone Burst :                Off\n");
	else
		printf("Tone Burst :                SAT%c\n",(toneburst - TONE_BURST_A) + 'A');
	if(diseqcport == DISEQC_DISABLE)
		printf("Diseqc Port :               Off\n");
	else
		printf("Diseqc Port :               %d\n",(diseqcport - DISEQC_PORT1) + 1);
	if(Sky_TunerLockTest(newLNB,newTR) != SKY_OK)
		Error("Sky_TunerLockTest Error");
	if(Sky_GetTunerInformation(&tunerResult))
		Error("Sky_GetTunerInformation Error");
	printf("Tuner Lock :                %s\n",tunerResult.TunerLock == 1 ? "Ok" : "Bad");
	printf("QPSK Lock :                 %s\n",tunerResult.QpskLock == 1 ? "Ok" : "Bad");
	printf("Viterbi Lock :              %s\n",tunerResult.ViterbiLock == 1 ? "Ok" : "Bad");
	printf("Frame Lock :                %s\n",tunerResult.FrameLock == 1 ? "Ok" : "Bad");
	printf("FEC Rate :                  %d/%d\n",tunerResult.FECRate,tunerResult.FECRate + 1);
	printf("AGC Value :                 %d\n",tunerResult.AGCValue);
	printf("Error 1 :                   %d\n",tunerResult.Error1);
	printf("Error 2 :                   %d\n",tunerResult.Error2);
	if(Sky_GetMacAddress(&macaddress))
		Error("Sky_GetMacAddress Error");
	ip.Ipaddress1 = ipaddr[0];
	ip.Ipaddress2 = ipaddr[1];
	ip.Ipaddress3 = ipaddr[2];
	ip.Ipaddress4 = ipaddr[3];
	if(Sky_SetIPAddress(ip))
		Error("Sky_SetIPAddress Error");
	ipormac = macfilter ? 1 : 0;
	if(Sky_SetIPorMACfiltering(&ipormac))
		Error("Sky_SetIPorMACfiltering Error");
	printf("MAC Address :               %02x:%02x:%02x:%02x:%02x:%02x\n",
				 macaddress.Macaddress1,macaddress.Macaddress2,macaddress.Macaddress3,
				 macaddress.Macaddress4,macaddress.Macaddress5,macaddress.Macaddress6);
	printf("IP Address :                %d.%d.%d.%d\n",ipaddr[0],ipaddr[1],ipaddr[2],ipaddr[3]);
	printf("Filtering :                 %s\n",macfilter ? "MAC Address" : "IP Address");
	for(i = 0;i < qtpids;i++)
	{
		pid.Encapsulation = datamode[i];
		pid.DataPID = datapid[i];
		pid.TransportID = 0;
		pid.pgnumber = 0;
		pid.PmtPID = 0;
		if(datamode[i] == 1)			strcpy(mode,"Data Piping");
		else if(datamode[i] == 2)	strcpy(mode,"Data Streaming");
		else if(datamode[i] == 3)	strcpy(mode,"Multiprotocol Encapsulation");
		else if(datamode[i] == 4)	strcpy(mode,"Data Carousel");
		else if(datamode[i] == 5)	strcpy(mode,"Sagem");
		else											strcpy(mode,"Unknown");
		printf("Data PID :                  %04X %s\n",datapid[i],mode);
		if(Sky_SetNetworkPID(pid))
			Error("Sky_SetNetworkPID Error");
	}
#if 0
	/* reinit multicast mode on eth0 */
	system("/sbin/ifconfig eth0 multicast");
	/* exit every 8 hours */
	sleep(28800);
#endif
	prevTunerStatus = 1;
	cntTime = 0;
	/* check the signal status at every five second.
	   And notify if there is change in tuner status. ... added by kdean */

	while(1)
	{
		if(Sky_GetTunerInformation(&tunerResult))
			Error("Sky_GetTunerInformation Error");
		curTunerStatus = tunerResult.TunerLock & tunerResult.QpskLock 
				& tunerResult.ViterbiLock & tunerResult.FrameLock;
		if(curTunerStatus != prevTunerStatus)
		{
			printf("Tuner %s !!!\n", strTunerStatus[curTunerStatus]);
			prevTunerStatus = curTunerStatus;
		}
		sleep(5);

	/* If your card stops working after a few hours , please uncomment followings 
		as the original code by Picciani did. */
	/*
		cntTime += 5;
		if(cntTime > 28800)
			break;
	*/
	}
 	
	for(i = 0;i < qtpids;i++)
	{
		pid.Encapsulation = datamode[i];
		pid.DataPID = datapid[i];
		pid.TransportID = 0;
		pid.pgnumber = 0;
		pid.PmtPID = 0;
		Sky_ReleaseNetworkPID(pid);
	}
	skyPIDs = 0;
	Sky_StopSystem();
	skyStart = 0;
	Sky_CloseSystem();
	skyOpen = 0;
	return(0);
}
