#ifndef _SKYMEDIA_H_
#define _SKYMEDIA_H_

#ifdef __cplusplus
extern "C" {			// Assume C declarations for C++
#endif	// __cplusplus

#ifndef __SM2_GLOB_H_
#define TRUE	1
#define FALSE	0
#define OK		0
#define ERROR	-1

#ifndef __KERNEL__
typedef unsigned int u32;
typedef unsigned short u16;
typedef unsigned char u8;
#endif
#endif

/* The size of arrays used in driver */
#define MAX_SAT_NAME_LEN       30
#define MAX_PSI_DVB_SI_NUM     9
#define MAX_AUDIO_PID          6
#define MAX_PROGRAM_NAME_LEN   31
#define MAX_PROGRAM_NUM        80
#define MAX_DATA_NUM           80
#define MAX_DATA_PID           16

/* Parameters for Sky_OpenSystem() */
#define MAIN_APP       1
#define REMOCON_APP    2
#define DATA_APP       3
#define FIRMWARE_APP   4

/* Return value of the Sky_TunerLockTest() */
#define SKY_OK         1
#define SKY_LOCK_ERROR 2
#define SKY_ERROR      3

/* FEC rate of the TUNER_RESULT structure */
#define TUNER_RATE12   1
#define TUNER_RATE23   2
#define TUNER_RATE34   3
#define TUNER_RATE45   4
#define TUNER_RATE56   5
#define TUNER_RATE67   6
#define TUNER_RATE78   7
#define TUNER_RATE89   8

/* TunerLock value of the TUNER_RESULT structure */
#define TUNER_LOCK    1
#define TUNER_UNLOCK  0

/* Mode value of the AUDIO_CONTROL structure. */
#define AUDIO_STEREO_MODE   0
#define AUDIO_MONO_MODE     1
#define AUDIO_LEFT_ONLY     2
#define AUDIO_RIGHT_ONLY    3

/* Mute value of the AUDIO_CONTROL structure. */
#define AUDIO_MUTE          1
#define AUDIO_UNMUTE        0

/* Return value of the File open function. */
#define FILE_OPEN_ERROR    0
#define FILE_IS_USING      1
#define FILE_OPEN_SUCCESS  2

/* The return value of the Sky_InitDataPath, the Sky_TermDataPath,
	Sky_RegisterDataPath and Sky_UnRegisterDataPath */
#define DATA_OK					1
#define DATA_INVALID_PARAM		2
#define DATA_INIT_FAIL			3
#define DATA_TERM_FAIL			4
#define DATA_NOT_INIT			5
#define DATA_DUPLICATE_INIT		6
#define DATA_PID_FAIL			7
	
/* status value of the callback that registered by Sky_RegisterDataPath*/
#define DATA_STREAM_OK        0
#define DATA_STREAM_REMAIN    1

/* The value received from the Remocon. */
#define POWER_ONOFF      1
#define CHANNEL_UP       2
#define CHANNEL_DOWN     3
#define VOLUME_UP        4
#define VOLUME_DOWN      5
#define MUTE_ONOFF       6
#define INFORMATION      7

/* ToneBurst value of the LNB_STRUCTURE . */
#define TONE_BURST_DISABLE  0
#define TONE_BURST_A        1
#define TONE_BURST_B        2

/* DiseqcPort value of the LNB_STRUCTURE. */
#define DISEQC_DISABLE      0
#define DISEQC_PORT1        1
#define DISEQC_PORT2        2
#define DISEQC_PORT3        3
#define DISEQC_PORT4        4
#define DISEQC_PORT5        5
#define DISEQC_PORT6        6
#define DISEQC_PORT7        7
#define DISEQC_PORT8        8
#define DISEQC_PORT9        9
#define DISEQC_PORT10       10
#define DISEQC_PORT11       11
#define DISEQC_PORT12       12
#define DISEQC_PORT13       13
#define DISEQC_PORT14       14
#define DISEQC_PORT15       15
#define DISEQC_PORT16       16

#define DATA_PIPING                  1 
#define DATA_STREAMING               2
#define MULTIPROTOCOL_ENCAPSULATION  3
#define DATA_CAROUSEL                4
#define DATA_STREAMING_PES_1         5

#define DATA_TP_PACKET              10
#define DATA_PES_PACKET				11
#define DATA_SECTION_PACKET			12

#define MANUAL_ADD            1
#define AUTOMATIC_ADD         0

typedef enum _LNB_HILOW {
	LNB_HIGH,
	LNB_LOW,
	LNB_LIMIT
} LNB_HILOW;

typedef enum _LNB_POWER {
	LNB_POWER_ON,
	LNB_POWER_OFF,
	LNB_POWER_LIMIT
} LNB_POWER;

typedef enum _SIGNAL_TYPE {
	SIGNAL_VERTICAL,
	SIGNAL_HORIZONTAL,
	SIGNAL_LIMIT
} SIGNAL_TYPE;

typedef enum _VIDEO_OUTMODE {
	PAL_MODE,
	NTSC_MODE,
	VIDEOMODE_LIMIT
} VIDEO_OUTMODE;

typedef enum _WAIT_FLAG {
	_WAIT,
	_NO_WAIT,
	_WAIT_LIMIT
} WAIT_FLAG;

typedef struct {
	u8 SatelliteName[MAX_SAT_NAME_LEN];
	u32 LocalOscillator;
	LNB_HILOW SwitchOnOff;
	LNB_POWER LNBPower;
	u8 ToneBurst;
	u8 DiseqcPort;
} LNB_STRUCTURE, *P_LNB_STRUCTURE;

typedef struct {
	u32 TransponderNumber;
	u32 DownlinkFrequency;
	u32 SymbolRate;
	SIGNAL_TYPE SignalType;
	u16 TransportID;
} TR_STRUCTURE, *P_TR_STRUCTURE;

typedef struct {
	u32 ProgramNumber;
	u16 VideoPID;
	u16 AudioPIDCount;
	u16 AudioPID[MAX_AUDIO_PID];
	u16 PcrPID;
	u16 TTxtPID;
	u16 pgnumber;
	u16 PmtPID;
	u16 Scrambled;        /* if 1 program is scrambled, else if 0 program is not scrambled */
	u16 TVorRadio;        /* if 1 program is TV, else if 0 program is radio */
	u8 ProgramName[MAX_PROGRAM_NAME_LEN];
	u8 ManualAdd;
} TVPROGRAM_STRUCTURE, *P_TVPROGRAM_STRUCTURE;

typedef struct {
	u32 DataNumber;
	u16 pgnumber;
	u16 PmtPID;
	u16 DataPID;
	u16 Encapsulation;
        u32 SetFlag;
	u8 ManualAdd;
} DATAPROGRAM_STRUCTURE, *P_DATAPROGRAM_STRUCTURE;

typedef struct {
	LNB_STRUCTURE LnbInfo;
	TR_STRUCTURE TrInfo;
	TVPROGRAM_STRUCTURE TvInfo;
} FAVORITE_STRUCTURE, *P_FAVORITE_STRUCTURE;

typedef struct {
	LNB_STRUCTURE LnbInfo;
	TR_STRUCTURE TrInfo;
	u32 NumOfProgram;
	TVPROGRAM_STRUCTURE TvInfo[MAX_PROGRAM_NUM];
	u32 NumOfData;
	DATAPROGRAM_STRUCTURE DataInfo[MAX_DATA_NUM];
} SI_STRUCTURE, *P_SI_STRUCTURE;

typedef struct sAudioControl{
	u8 Volume;
	u8 Mute;
	u8 Mode;
} AUDIO_CONTROL, *P_AUDIO_CONTROL;

typedef struct sTunerResult
{
	u8 TunerLock;      // 1 : Lock   2: Unlock
	u8 QpskLock;
	u8 ViterbiLock;
	u8 FrameLock;
	u8 FECRate;        // 1: 1/2, 2: 2/3, 3: 3/4, 4: 4/5, 5: 5/6, 6: 6/7, 7: 7/8, 8: 8/9
	u8 AGCValue;
	u8 Error1;
	u8 Error2;
} TUNER_RESULT, *P_TUNER_RESULT;

typedef struct sIPAddress
{
	u8 Ipaddress1;
	u8 Ipaddress2;
	u8 Ipaddress3;
	u8 Ipaddress4;
} IP_ADDRESS, *P_IP_ADDRESS;

typedef struct sMACAddress
{
	u8 Macaddress1;
	u8 Macaddress2;
	u8 Macaddress3;
	u8 Macaddress4;
	u8 Macaddress5;
	u8 Macaddress6;
} MAC_ADDRESS, *P_MAC_ADDRESS;

typedef struct sVersionNum
{
	u8 HighVersion;
	u8 LowVersion;
} VERSION_NUM, *P_VERSION_NUM;

typedef u8 IP_OR_MAC_FILTERING, *P_IP_OR_MAC_FILTERING;

typedef u8 VIDEO_MODE, *P_VIDEO_MODE;

typedef struct {
	u16 VideoPID;
	u16 AudioPID;
	u16 PcrPID;
	u16 TransportID;
	u16 pgnumber;
	u16 PmtPID;
} SETCHANNEL_STRUCTURE, *P_SETCHANNEL_STRUCTURE;

typedef struct {
	u16 DataPID;
	u16 Encapsulation;
	u16 TransportID;
	u16 pgnumber;
	u16 PmtPID;
} SETDATA_STRUCTURE, *P_SETDATA_STRUCTURE;

/* Initialization and close functions */
/* this is a value of flag parameter
#define MAIN_APP       1
#define REMOCON_APP    2
#define DATA_APP       3
#define FIRMWARE_APP   4
*/
int Sky_OpenSystem(u8 Flag);
int Sky_CloseSystem(void);
int Sky_StartSystem(void);
int Sky_StopSystem(void);
/* this is return value of below function
#define SKY_OK         1
#define SKY_LOCK_ERROR 2
#define SKY_ERROR      3
*/
int Sky_TunerLockTest(LNB_STRUCTURE lnblist, TR_STRUCTURE trlist);
int Sky_SetChannel(SETCHANNEL_STRUCTURE channel_list);
int Sky_StopChannel(SETCHANNEL_STRUCTURE channel_list);
int Sky_SetAudioChannel(u16 AudioPID);
int Sky_GetTunerInformation(P_TUNER_RESULT pTunerResult);
int Sky_SetAudioControl(AUDIO_CONTROL AudioControl);
int Sky_SearchCancel(void);

int Sky_ChannelFileOpen(char *FileName);
int Sky_ChannelFileCreate(char *FileName);
int Sky_ChannelFileClose(void);
int Sky_ChannelFileRead(void);
int Sky_ChannelFileWrite(void);

int Sky_FavoriteFileOpen(char *FileName);
int Sky_FavoriteFileCreate(char *FileName);
int Sky_FavoriteFileClose(void);
int Sky_FavoriteWriteFile(void);
int Sky_FavoriteReadFile(void);

int Sky_GetSatelliteInformation(LNB_STRUCTURE *lnbinfo);
int Sky_SetNewLNBInformation(LNB_STRUCTURE lnbinfo);

int Sky_AddTransponder(TR_STRUCTURE *trlist);
int Sky_UpdateTransponder(TR_STRUCTURE trlist);
int Sky_DeleteTransponder(u32 TransponderNumber);
int Sky_GetInfoTransponder(u32 TransponderNumber, TR_STRUCTURE *trlist);
u32 Sky_GetNumOfTransponder(void);

u32 Sky_GetNumOfProgram(u32 TransponderNumber);
u32 Sky_GetNumOfData(u32 TransponderNumber);
int Sky_GetInfoProgram(TVPROGRAM_STRUCTURE *tvlist);
int Sky_GetInfoData(DATAPROGRAM_STRUCTURE *datalist);
int Sky_AddProgram(u32 TransponderNumber, TVPROGRAM_STRUCTURE *tvlist);
int Sky_AddData(u32 TransponderNumber, DATAPROGRAM_STRUCTURE *datalist);
int Sky_DeleteProgram(TVPROGRAM_STRUCTURE tvlist);
int Sky_DeleteData(DATAPROGRAM_STRUCTURE datalist);
int Sky_UpdateProgram(TVPROGRAM_STRUCTURE tvlist);
int Sky_UpdateData(DATAPROGRAM_STRUCTURE datalist);

int Sky_DeleteFavorite(FAVORITE_STRUCTURE falist);
int Sky_AddFavorite(FAVORITE_STRUCTURE *falist);
int Sky_UpdateFavorite(u32 index, FAVORITE_STRUCTURE new_falist);
int Sky_GetNextFavorite(FAVORITE_STRUCTURE curlist, FAVORITE_STRUCTURE *falist);
int Sky_GetPrevFavorite(FAVORITE_STRUCTURE curlist, FAVORITE_STRUCTURE *falist);
int Sky_GetFirstProgramInFavorite(FAVORITE_STRUCTURE *falist);
u32 Sky_GetNumOfFavorite(void);
int Sky_GetFavoriteInfo(u32 FavoriteNumber, FAVORITE_STRUCTURE *falist);

int Sky_FreeTransponderStructure(void);

int Sky_SetNetworkPID(SETDATA_STRUCTURE datalist);
int Sky_ReleaseNetworkPID(SETDATA_STRUCTURE datalist);
int Sky_ReleaseAllNetworkPID(void);
int Sky_SetIPAddress(IP_ADDRESS IpAddress);
int Sky_GetMacAddress(P_MAC_ADDRESS lpMacAddress);
int Sky_SetIPorMACfiltering(P_IP_OR_MAC_FILTERING pIPorMACfiltering);

int Sky_GetVideoMode(P_VIDEO_MODE pVideoMode);
int Sky_GetFirmwareVersion(P_VERSION_NUM pVersionNumber);
int Sky_GetDriverVersion(P_VERSION_NUM pVersionNumber);

int Sky_SearchChannel(WAIT_FLAG wait, SI_STRUCTURE *si_structure);

int Sky_LoadFirmware( char *file_name );

#if defined(SM200D) || defined(SM300)
int Sky_GetFirmFileVersion(char* filename, P_VERSION_NUM pVersionNumber);
#endif
// defined in sm2_sys.c
void Sleep( u32 );

#ifdef __cplusplus
}		// End of extern "C" {
#endif	// __cplusplus

#endif

